package mp3tools;

public class Header {

	//TODO: Add throws IllegalHeaderException to creation of a header class, since header must
	//Read data from header, it must be in correct format!
	//Also implement try/catch blocks in all class which create Header objects!!
	
	private int h_version, h_layer, h_sampRate, h_bitrate;
	private int h_frameLength;
	private int h_protectionBit;
	private int h_paddingBit;
	private int h_privateBit;
	private int h_channelMode;
	private int h_modeExtension;
	private int h_copyright;
	private int h_original;
	private int h_emphasis;
	private byte[] headerData;
	
	//MPEG Version specs.
	//Offset 0x11
	public static final int		MPEG2_POINT5 = 0;
	public static final int		MPEG_RESERVED = 1;
	public static final int		MPEG2 = 2;
	public static final int		MPEG1 = 3;
	
	//MPEG LAYER specs.
	//Offset 0x13
	public static final int		LAYER_RESERVED = 0;
	public static final int 	LAYER3 = 1;
	public static final int 	LAYER2 = 2;
	public static final int		LAYER1 = 3;
	
	//Protection specs.
	//Offset 0x15
	public static final int		PROTECTED = 0;
	public static final int		NOT_PROTECTED = 1;
	
	//Bitrate specs.
	//Offset 0x16
	//[h_version][h_layer][h_bitrate]
	public static final int[][][] BITRATE_INDEX = 		
		   //V2.5: Reserved, L3, L2, L1
	      {{{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
			{0 /*free*/, 8000, 16000, 24000, 32000, 40000, 48000,
		56000, 64000, 80000, 96000, 112000, 128000, 144000, 160000, 0 /*bad*/},
			{0 /*free*/, 8000, 16000, 24000, 32000, 40000, 48000,
		56000, 64000, 80000, 96000, 112000, 128000, 144000, 160000, 0 /*bad*/},
		    {0 /*free*/, 32000, 48000, 56000, 64000, 80000, 96000,
		112000, 128000, 144000, 160000, 176000, 192000 ,224000, 256000, 0 /*bad*/}},


			//Reserved: Don't know bitrate, keep zero;
			//Reserved: Reserved, L3, L2, L1
		   {{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
			{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
			{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
			{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}},
			  
		   //V2: Reserved, L3, L2, L1
		   {{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
			{0 /*free*/, 8000, 16000, 24000, 32000, 40000, 48000,
		56000, 64000, 80000, 96000, 112000, 128000, 144000, 160000, 0 /*bad*/},
			{0 /*free*/, 8000, 16000, 24000, 32000, 40000, 48000,
		56000, 64000, 80000, 96000, 112000, 128000, 144000, 160000, 0 /*bad*/},
			{0 /*free*/, 32000, 48000, 56000, 64000, 80000, 96000,
		112000, 128000, 144000, 160000, 176000, 192000 ,224000, 256000, 0 /*bad*/}},
		  
		   //V1: Reserved, L3, L2, L1
		   {{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
		    {0 /*free*/, 32000, 40000, 48000, 56000, 64000, 80000,
		96000, 112000, 128000, 160000, 192000, 224000, 256000, 320000, 0 /*bad*/},
			{0 /*free*/, 32000, 48000, 56000, 64000, 80000, 96000,
   	    112000, 128000, 160000, 192000, 224000, 256000, 320000, 384000, 0 /*bad*/},
		    {0 /*free*/, 32000, 64000, 96000, 128000, 160000, 192000,
		224000, 256000, 288000, 320000, 352000, 384000, 416000, 448000, 0 /*bad*/}},
		   //If bitrate is ever zero, we should call an error.
		   //Sinc Free format is too much work to deal with.
		   //TODO: Add VBR and Xing support.
		  };
	
	//Samp Rate specs [h_version][h_sampRate]
	//Offset 0x20
	public static final int[][] SAMPLING_FREQUENCY_INDEX = 
	//V2.5
	   {{11025, 12000, 8000, 0 /*reserved*/},
	//VReserved
		{0, 0, 0, 0},
	//V2
		{22050, 24000, 16000, 0 /*reseved*/},
	//V1
		{44100, 48000, 32000, 0 /*reserved*/}
	};
	
	//Padding Bit specs
	//Offset 0x22
	public static final int	 NOT_PADDED = 0;
	public static final int  PADDED = 1;
	
	//Private Bit;
	//Offset 0x23
	
	//Channel Mode specs
	//OFfset 0x24
	public static final int STEREO = 0;
	public static final int JOINT_STEREO = 1;
	public static final int DUAL_CHANNEL = 2;
	public static final int SINGLE_CHANNEL = 3;
	
	//Mode Extension specs (Only applicable if h_channelMode == JOINT_STEREO)
	//Offset 0x26
	//TODO: Only needed for encoding/decoding
	
	//Copyright specs
	//Offset 0x28
	public static final int NOT_COPYRIGHTED = 0;
	public static final int COPYRIGHTED = 1;
	
	//Original specs
	//Offset 0x29
	public static final int NOT_ORIGINAL = 0; //Frame is copy
	public static final int ORIGINAL = 1;
	
	//Emphasis specs
	//Offset 0x30
	//TODO: No idea what this is for, don't worry about.
	
	public static enum mpegVersion 		{MPEG1, MPEG2, MPEG2POINT5, RESERVED};
	public static enum layerDescription {LAYERI, LAYERII, LAYERIII, RESERVED};
	public static enum sampRate 		{FOURTYFOURPOINTONE, FOURTYEIGHT, THIRTYTWO, TWENTYTWOPOINTZEROFIVE, TWENTYFOUR, SIXTEEN, ELEVENPOINTZEROTWOFIVE, TWELVE, EIGHT, RESERVED};
	//public static enum bitrateIndex		{FREE, EIGHT, SIXTEEN, TWENTYFOUR, THIRTYTWO, FOURTY, FOURTYEIGHT, FIFTYSIX, SIXTYFOUR, EIGHTY, NINETYSIX, ONEHUNDREDTWELVE, ONEHUNDREDTWENTYEIGHT, ONEHUNDREDFOURTYFOUR, ONEHUNDREDSIXTY, 
	
	
	/**
	 * Constructs new frame.
	 * @return 
	 * @throws IllegalHeaderException 
	 */
	public Header(byte[] newHeaderData) throws IllegalHeaderException {
		//First check if header is sync.
		headerData = new byte[newHeaderData.length];
		//Copies data from input into private variable. Use arraycopy to copy data, not pointer.
		System.arraycopy(newHeaderData, 0, headerData, 0, newHeaderData.length);
		if (!isHeader()) throw new IllegalHeaderException();
		/*int returnValue = */readHeader();
		//If header is garbage, return garbage.
		//if (returnValue == 0) frameData = new byte[0];
		
	}
	
	public boolean isHeader() {
		if (!(((headerData[0] & 255) == 255) && (((headerData[1] >>> 5) & 7) == 7))) return false;
		else return true;
	}
	/*
	public static boolean isHeader(byte[] myHeaderData) {
		if (!(((myHeaderData[0] & 255) == 255) && (((myHeaderData[1] >>> 5) & 7) == 7))) return false;
		else return true;
	}
	*/
	
	private void readHeader() {
		// This function reads the header of the frame and sets all the internal variables.
		byte[] header = new byte[4]; //32 bits long.
		System.arraycopy(headerData, 0, header, 0, 4);
		//Check if this is a frame (eg: Check frame sync.)
		//if (!(((header[0] & 255) == 255) && (((header[1] >>> 5) & 7) == 7)))
		//Read Version
		h_version = (header[1] >>> 3) & 3;
		//Read Layer
		h_layer = (header[1] >>> 1) & 3;
		//Read Protected
		h_protectionBit = (header[1] & 1);
		//Read Bitrate Index
		h_bitrate = (header[2] >>> 4) & 15;
		//Read Sampling Freq Index
		h_sampRate = (header[2] >>> 2) & 3;
		//Read Padding Bit
		h_paddingBit = (header[2] >>> 1) & 1;
		//Read Private Bit
		h_privateBit = (header[2] & 1);
		//Read Channel Mode
		h_channelMode = (header[3] >>> 6) & 3;
		//Read Mode Extension
		h_modeExtension = (header[3] >>> 4) & 3;
		//Read CopyRight
		h_copyright = (header[3] >>> 3) & 1;
		//Read original
		h_original = (header[3] >>> 2) & 1;
		//Read emphasis
		h_emphasis = (header[3] & 3);
		//return 1;
		
		
	}

	/**
	 * 
	 * @return Returns Frame Length
	 */
	public int getFrameLength()
	{
		//Layer I
		if (h_layer == 3)
		{
			//12 * BitRate / SampleRate + Padding) * 4 

		}
		//Layer II, III
		if (h_layer == 2 || h_layer == 1)
		{
			return (144 * BITRATE_INDEX[h_version][h_layer][h_bitrate] / SAMPLING_FREQUENCY_INDEX[h_version][h_sampRate]) + h_paddingBit;
		}
		else return 0;
		//Reserved...
	}
	
	/**
	 * 
	 * @return Returns Sample Rate in Hz
	 */
	public int getSampRate()
	{
		return SAMPLING_FREQUENCY_INDEX[h_version][h_sampRate];
	}
	
	public String getMpegVersion()
	{
		String version = "";
		switch (h_version)
		{
			case (MPEG2_POINT5): version = "MPEG-2.5"; break;
			case (MPEG_RESERVED): version = "MPEG-RESERVED"; break;
			case (MPEG2): version = "MPEG-2"; break;
			case (MPEG1): version = "MPEG-1"; break;
		
		}
		return version;
	}
	public String getLayerDescription()
	{
		String layer = "";
		switch (h_layer)
		{
			case (LAYER_RESERVED): layer = "Layer Reserved"; break;
			case (LAYER3): layer = "Layer 3"; break;
			case (LAYER2): layer = "Layer 2"; break;
			case (LAYER1): layer = "Layer 1"; break;
		}
		return layer;
	}
	
	/**
	 * 
	 * @return Returns Mpeg Version (either 0, 1, 2 or 2.5)
	 */
	//public int getMpegVersion()
	//{
	//	switch h_version
	//}
	
	/**
	 * 
	 * @return Returns Layer Description (either I, II, or III)
	 */
	//public int getLayerDescription()
	//{
	//	
	//}
	
	/**
	 * 
	 * @return Returns true if protected by CRC
	 */
	//public boolean isProtected()
	//{
	//	
	//}
	
	/**
	 * 
	 * @return Returns Bitrate in bps.
	 */
	//public int getBitrateIndex()
	//{
		
	//}
	
	/**
	 * 
	 * @return Returns true if padded.
	 */
	//public boolean isPadded()
	//{
		
	//}
}